# Setup file for BVAR 2 analyses
#
# Patrick T. Brandt
# pbrandt@utdallas.edu
#
#
# Original version: April 18, 2006
# Revisions       : August 15, 2006 -- changed the SVAR structure to
#                   allow shocks in MP to affect Information
#
# Sets up the common variables, data and model pieces needed for
# running the two models in the paper.

library(MSBVAR)
library(foreign)

# Read in the macro data and format for analysis
tmp <- read.dta("BVAR2MacroData.dta")
attach(tmp)
# Set up matrix of the endogenous variables
Y <- ts(cbind(log(crbspot), log(m2sl), 0.01*fedfunds, log(rgdpm),
              log(cpiaucsl), 0.01*unrate, 0.01*mics, approvei,
              mpi), start=c(1959,1), freq=12)

# Set up exogenous variables
# recode Dem POTUS to do sign flips
demparty <- ifelse(demparty==0, -1, 1)
Z <- ts(cbind(hnymoon, demparty, eleccount), start=c(1959,1), freq=12)

# Clean up some of the data input stuff.
detach(tmp)
rm(tmp)

# Subset the valid data for the sample and set up variable names
Y <- window(Y, start=c(1978,1), end=c(2004,6))
X <- window(Z, start=c(1978,1), end=c(2004,6))

colnames(Y) <- c("Pcom", "M2", "R",
                 "Y", "CPI", "U",  "CS",
                 "Approval", "MP")

colnames(X) <- c("Honeymoon", "Presparty", "Eleccounter")


# Set up identification matrices
m <- ncol(Y)
general.A0 <- matrix(0, m, m)
diag(general.A0) <- 1
colnames(general.A0) <- c("Info", "MP", "MD", "Prod1", "Prod2",
                          "Prod3",
                          "Macropolity1", "Macropolity2",
                          "Macropolity3")

rownames(general.A0) <- colnames(Y)

# Remember that columns are equations, so we can put in the 1-0
# combinations for the variables than hit each equation.
general.A0[1:9,1] <- 1
general.A0[c(2,3,8),2] <- 1
general.A0[c(2:5,8),3] <- 1
general.A0[4,4] <- 1
general.A0[4:5,5] <- 1
general.A0[4:6,6] <- 1
general.A0[4:7,7] <- 1
general.A0[4:8,8] <- 1
general.A0[4:9,9] <- 1

# Now set up the lags and monthly decay structures
p <- 13
qm <- 12

# Set parameters for the draws

N1 <- 2000
N2 <- 20000       # posterior sample
thin.iter <- 1    # thinning interval

# Wrapper function for calling parallel chains.

parallel.model <- function(i, fit.model, N1, N2, thin) {
    require(MSBVAR)
    m <- nrow(fit.model$A0.mode)
    # Need to over disperse the A0 values for the algorithm
    fit.model$A0.mode <- normalize.svar(fit.model$A0.mode +
                                        5*diag(rnorm(m)),
                                        fit.model$A0.mode,
                                        method="DistanceMLA")$A0normalized

    gibbs.A0(varobj=fit.model, N1=N1, N2=N2,
             thin=thin, normalization="DistanceMLA")

}

# Transform the A0's into a set of mcmc chains of the free parameters
# for posterior convergence analyses.

A0.chains2mcmc.list <- function(chains)
{ num.chains <- length(chains)

  chain.out <- vector(mode="list", length=num.chains)

  for (i in 1:num.chains)
  {
      chain.out[[i]] <- A02mcmc(chains[[i]])
  }

  return(mcmc.list(chain.out))
}


# Function to combine A0 draws from multiple chains

A0parallel2A0obj <- function(chains)
{ num.chains <- length(chains)
  A0.flat <- chains[[1]]$A0.posterior$A0
  A0.struct <- chains[[1]]$A0.posterior$struct
  m <- chains[[1]]$A0.posterior$m

  W.flat <- chains[[1]]$W.posterior$W
  W.index <- chains[[1]]$W.posterior$W.index

  for(i in 2:num.chains)
  {
      A0.flat <- c(A0.flat, chains[[i]]$A0.posterior$A0)
      W.flat <- c(W.flat, chains[[i]]$W.posterior$W)
      update.index <- max(W.index) + chains[[i]]$W.posterior$W.index
      W.index <- c(W.index, update.index)
  }

  A0.posterior <- list(A0=A0.flat,struct=A0.struct,m=m)
  W.posterior <- list(W=W.flat, W.index=W.index, m=m)

  return(list(A0.posterior=A0.posterior,
              W.posterior=W.posterior,
              ident=chains[[1]]$ident,
              thin=chains[[1]]$thin,
              N2=num.chains*chains[[1]]$N2))
}


# Save results
save.image("Data_Setup.RData")
q(save="no")
